import { Card, CardContent, CardDescription, CardHeader, CardTitle } from "@/components/ui/card";
import { useGalleryDoc, useGalleryDocSimilar, useRehashDoc, useRescanDoc } from "../hook/useGalleryDoc.ts";
import TagBadge from "@/components/gallery/TagBadge";
import StyledLink from "@/components/gallery/StyledLink";
import { Link, useLocation } from "wouter";
import { classifyTags } from "../lib/classifyTags.tsx";
import { DescTagItem, DescItem } from "../components/gallery/DescItem.tsx";
import { Button } from "@/components/ui/button.tsx";
import { GalleryCard } from "@/components/gallery/GalleryCard.tsx";
import { useEffect, useRef } from "react";
import { useLogin } from "@/state/user.ts";

export interface ContentInfoPageProps {
    params: {
        id: string;
    };
}

function Wrapper({ children }: { children: React.ReactNode }) {
    const ref = useRef<HTMLDivElement>(null);
    const [pathname] = useLocation();
    useEffect(() => {
        if (ref.current) {
            ref.current.scrollTo({
                top: 0,
                left: 0,
            });
        }
    }, [pathname]);

    return <div className="p-4 overflow-auto h-dvh" ref={ref}>
        {children}
    </div>;
}

export function ContentInfoPage({ params }: ContentInfoPageProps) {
    const { data, error, isLoading } = useGalleryDoc(params.id);
    const rehashDoc = useRehashDoc();
    const rescanDoc = useRescanDoc();
    const user = useLogin();
    const username = user?.username;
    const isAdmin = username === "admin";

    if (isLoading) {
        return <div className="p-4 flex items-center justify-center h-full">
            <span className="animate-pulse text-4xl">
                Loading...
            </span>
        </div>
    }

    if (error) {
        return <div className="p-4">Error: {String(error)}</div>
    }

    if (!data) {
        return <div className="p-4">Not found</div>
    }

    const tags = data?.tags ?? [];
    const classifiedTags = classifyTags(tags);

    const contentLocation = `/doc/${params.id}/reader`;

    return (
        <Wrapper>
            <Link to={contentLocation}>
                <div className="m-auto h-[400px] mb-4 flex justify-center items-center flex-none bg-[#272733]
            rounded-xl shadow-lg overflow-hidden">
                    <img
                        className="max-w-full max-h-full object-cover object-center"
                        src={`/api/doc/${data.id}/comic/thumbnail`}
                        alt={data.title} />
                </div>
            </Link>
            <Card className="flex-1 relative">
                {isAdmin &&
                    <div className="absolute top-0 right-0 p-2 grid">
                        <Button variant="ghost" onClick={async () => {
                            // Rehash
                            await rehashDoc(params.id);
                        }}>Rehash</Button>
                        <Button variant="ghost" onClick={async () => {
                            if (!window.confirm("Are you sure?")) {
                                return;
                            }
                            // Rescan
                            await rescanDoc(params.id);
                        }}>Rescan</Button>
                    </div>
                }
                <CardHeader>
                    <CardTitle>
                        <StyledLink to={contentLocation}>
                            {data.title}
                        </StyledLink>
                    </CardTitle>
                    <CardDescription>
                        <StyledLink to={`/search?allow_tag=type:${classifiedTags.type[0] ?? ""}`} className="text-sm">
                            {classifiedTags.type[0] ?? "N/A"}
                        </StyledLink>
                    </CardDescription>
                </CardHeader>
                <CardContent>
                    <div className="grid gap-4 grid-cols-[repeat(auto_fill_300px)]">
                        <DescTagItem name="Artist" items={classifiedTags.artist} />
                        <DescTagItem name="Group" items={classifiedTags.group} />
                        <DescTagItem name="Series" items={classifiedTags.series} />
                        <DescTagItem name="Character" items={classifiedTags.character} />
                        <DescItem name="Created At / Modified At">{new Date(data.created_at).toLocaleString()} / {new Date(data.modified_at).toLocaleString()}</DescItem>
                        <DescItem name="Filehash">{data.content_hash}</DescItem>
                        <DescItem name="Path">{`${data.basepath}/${data.filename}`}</DescItem>
                        <DescItem name="Page Count">{data.pagenum}</DescItem>
                    </div>
                    <div className="grid mt-4">
                        <span className="text-muted-foreground text-sm">Tags</span>
                        <ul className="mt-2 flex flex-wrap gap-1">
                            {classifiedTags.rest.map((tag) => <TagBadge key={tag} tagname={tag} />)}
                        </ul>
                    </div>
                </CardContent>
            </Card>
            <SimilarContentCard id={params.id} />
        </Wrapper>
    );
}

function SimilarContentCard({
    id,
}: {
    id: string;
}) {
    const { data, error, isLoading } = useGalleryDocSimilar(id);

    if (isLoading) {
        return <div className="p-4">Loading...</div>
    }

    if (error) {
        return <div className="p-4">Error: {String(error)}</div>
    }

    if (!data) {
        return <div className="p-4">Not found</div>
    }

    return (
        <div className="space-y-4 mt-4 mx-2">
            <h2 className="text-2xl font-bold">Contents with Similar Tags</h2>
            <div className="grid gap-4 grid-cols-[repeat(auto_fill_300px)]">
                {data.map((doc) => (
                    <GalleryCard key={doc.id} doc={doc} />
                ))}
            </div>
        </div>
    )
}

export default ContentInfoPage;